@@ -4,8 +4,7 @@ package Sidef::Deparse::Perl {
     use 5.014;
 
     use List::Util qw(all);
-    use File::Basename qw(dirname);
-    use Scalar::Util qw(refaddr reftype);
+    use Scalar::Util qw(refaddr);
 
     my %addr;
     my %type;
@@ -53,7 +52,7 @@ package Sidef::Deparse::Perl {
                                    'Sidef::Types::Block::For'   => 1,
                                   },
 
-            reassign_ops => {map (("$_=" => $_), qw(+ - % * / & | ^ ** && || << >> ÷))},
+            reassign_ops => {map (("$_=" => $_), qw(+ - % * // / & | ^ ** && || << >> ÷))},
 
             inc_dec_ops => {
                             '++' => 'inc',
@@ -111,7 +110,6 @@ HEADER
         my ($self, $obj) = @_;
 
         my $ref = ref($obj);
-
         $self->_dump_string(
                               $ref eq 'Sidef::Variable::ClassInit'    ? $self->_dump_class_name($obj)
                             : $ref eq 'Sidef::Variable::Ref'          ? 'REF'
@@ -180,7 +178,7 @@ HEADER
             ref($var) || next;
             if (exists $var->{array}) {
                 my $name = $var->{name} . refaddr($var);
-                push @{$self->{function_declarations}}, [-1, 'my @' . $name . ';'];
+                push @{$self->{block_declarations}}, [$self->{current_block} // -1, 'my @' . $name . ';'];
 
                 # Overwrite with the default values, when the array is empty
                 if (exists $var->{value}) {
@@ -197,7 +195,7 @@ HEADER
             }
             elsif (exists $var->{hash}) {
                 my $name = $var->{name} . refaddr($var);
-                push @{$self->{function_declarations}}, [-1, 'my %' . $name . ';'];
+                push @{$self->{block_declarations}}, [$self->{current_block} // -1, 'my %' . $name . ';'];
 
                 # Overwrite with the default values, when the hash has no keys
                 if (exists $var->{value}) {
@@ -220,8 +218,8 @@ HEADER
             }
         }
 
-        # XXX: should we store the declaration in something like 'block_declarations'?
-        push @{$self->{function_declarations}}, [-1, 'my(' . join(', ', map { $self->_dump_var($_) } @vars) . ')' . ';'];
+        push @{$self->{block_declarations}},
+          [$self->{current_block} // -1, 'my(' . join(', ', map { $self->_dump_var($_) } @vars) . ')' . ';'];
 
         # Return the variables on assignments
         if (@code > 1 or exists($init_obj->{args})) {
@@ -234,8 +232,9 @@ HEADER
     }
 
     sub _dump_sub_init_vars {
-        my ($self, @vars) = @_;
+        my ($self, %opt) = @_;
 
+        my @vars = @{$opt{vars}};
         @vars || return '';
 
         my @dumped_vars = map { ref($_) ? $self->_dump_var($_) : $_ } @vars;
@@ -245,50 +244,56 @@ HEADER
             return '';
         }
 
-        #my $code = (' ' x $Sidef::SPACES) . "my (" . join(', ', @dumped_vars) . ') = @_;' . "\n";
-        my $code = (' ' x $Sidef::SPACES) . join(
-            "\n" . (' ' x $Sidef::SPACES),
-            join(
-                '',
-                map { s/^\s+//r }
-                  "my \@__vars__ = (" . join(', ', map { $_ eq 'undef' ? $_ : "\\my $_" } @dumped_vars) . ');',
-                "{state \$table = {"
-                  . join(', ', map { ref($vars[$_]) ? "$vars[$_]{name} => $_" : () } 0 .. $#vars) . "};",
-                'my (@left, %seen);',
-                q(foreach my $arg(@_) {),
-                q(    if (ref($arg) eq 'Sidef::Variable::NamedParam') {),
-                q(        if (exists $table->{$arg->[0]}) {),
-                q(            my $var = $__vars__[$table->{$arg->[0]}] // next;),
-                q(            if (ref($var) eq 'SCALAR') { ),
-                q(               $$var = ${$arg->[1]}[-1];),
-                q(            } elsif (ref($var) eq 'ARRAY') {),
-                q(              @$var = @{$arg->[1]};),
-                q(            } else {),
-                q(              %$var = @{$arg->[1]};),
-                q(            }),
-                q(            undef $seen{$var};),
-                q(         } else {),
-                q(             die "No such named parameter: <<$arg->[0]>>";),
-                q(         }),
-                q(     } else {),
-                q(        push @left, $arg;),
-                q(     }),
-                q(}),
+        my $code;
 
-                q(foreach my $var(@__vars__) {),
-                q(    next if exists $seen{$var // do{shift @left; next}};),
-                q(    @left || last;),
-                q(    if (ref($var) eq 'SCALAR') {),
-                q(      $$var = shift @left;),
-                q(    } elsif (ref($var) eq 'ARRAY') {),
-                q(      @$var = @left; last;),
-                q(    } else {),
-                q(      %$var = @left; last;),
-                q(    }),
-                q(}}),
-                )
-          )
-          . "\n";
+        if ($opt{extended}) {
+            $code = (' ' x $Sidef::SPACES) . join(
+                "\n" . (' ' x $Sidef::SPACES),
+                join(
+                    '',
+                    map { s/^\s+//r }
+                      "my \@__vars__ = (" . join(', ', map { $_ eq 'undef' ? $_ : "\\my $_" } @dumped_vars) . ');',
+                    "{state \$table = {"
+                      . join(', ', map { ref($vars[$_]) ? "$vars[$_]{name} => $_" : () } 0 .. $#vars) . "};",
+                    'my (@left, %seen);',
+                    q(foreach my $arg(@_) {),
+                    q(    if (ref($arg) eq 'Sidef::Variable::NamedParam') {),
+                    q(        if (exists $table->{$arg->[0]}) {),
+                    q(            my $var = $__vars__[$table->{$arg->[0]}] // next;),
+                    q(            if (ref($var) eq 'SCALAR') { ),
+                    q(               $$var = ${$arg->[1]}[-1];),
+                    q(            } elsif (ref($var) eq 'ARRAY') {),
+                    q(              @$var = @{$arg->[1]};),
+                    q(            } else {),
+                    q(              %$var = @{$arg->[1]};),
+                    q(            }),
+                    q(            undef $seen{$var};),
+                    q(         } else {),
+                    q(             die "No such named parameter: <<$arg->[0]>>";),
+                    q(         }),
+                    q(     } else {),
+                    q(        push @left, $arg;),
+                    q(     }),
+                    q(}),
+
+                    q(foreach my $var(@__vars__) {),
+                    q(    next if exists $seen{$var // do{shift @left; next}};),
+                    q(    @left || last;),
+                    q(    if (ref($var) eq 'SCALAR') {),
+                    q(      $$var = shift @left;),
+                    q(    } elsif (ref($var) eq 'ARRAY') {),
+                    q(      @$var = @left; last;),
+                    q(    } else {),
+                    q(      %$var = @left; last;),
+                    q(    }),
+                    q(}}),
+                    )
+              )
+              . "\n";
+        }
+        else {
+            $code = (' ' x $Sidef::SPACES) . "my (" . join(', ', @dumped_vars) . ') = @_;' . "\n";
+        }
 
         foreach my $var (@vars) {
 
@@ -502,19 +507,8 @@ HEADER
 
                     # Check the return value (when "-> Type" is specified)
                     if (exists $obj->{returns}) {
-
-                        my $type = $self->_dump_reftype($obj->{returns});
-
-                        $code =
-                            "do { $code;\n"
-                          . (' ' x $Sidef::SPACES)
-                          . "my \$_$refaddr = \$$obj->{name}$refaddr;\n"
-                          . (' ' x $Sidef::SPACES)
-                          . "\$$obj->{name}$refaddr = Sidef::Types::Block::Code->new("
-                          . "sub {my \$arg = \$_$refaddr->call(\@_); "
-                          . "(ref(\$arg) eq $type || ($type ne 'REF' && eval{\$arg->SUPER::isa($type)})) or "
-                          . " die q{[ERROR] Invalid return-type from $obj->{type} $self->{class_name}<<$name>>: got <<}"
-                          . " . ref(\$arg) . q{>>, but expected $type}; \$arg })}";
+                        my $types = '[' . join(',', map { $self->_dump_reftype($_) } @{$obj->{returns}}) . ']';
+                        $code = "do {$code; \$$obj->{name}$refaddr\->{returns} = $types; \$$obj->{name}$refaddr}";
                     }
 
                     # Memoize the method/function (when "is cached" trait is specified)
@@ -523,7 +517,7 @@ HEADER
                         $code =
                             "do {$code;\n"
                           . (' ' x $Sidef::SPACES)
-                          . "\$$obj->{name}$refaddr = Sidef::Types::Block::Code->new(Memoize::memoize(\$$obj->{name}${refaddr}->{code}))}";
+                          . "\$$obj->{name}$refaddr\->{code} = Memoize::memoize(\$$obj->{name}${refaddr}->{code}); \$$obj->{name}$refaddr}";
                     }
 
                     if ($obj->{type} eq 'method') {
@@ -566,60 +560,15 @@ HEADER
                 }
             }
         }
-        elsif ($ref eq 'Sidef::Variable::Struct') {
-            my $name = $self->_dump_class_name($obj);
-            if ($addr{$refaddr}++) {
-                $code = $name;
-            }
-            else {
-                local $self->{parent_name} = ['struct initialization', $obj->{name}];
-
-                # Mark the variables all in use
-                foreach my $var (@{$obj->{vars}}) {
-                    $var->{in_use} = 1;
-                }
-
-                $Sidef::SPACES += $Sidef::SPACES_INCR;
-                $code =
-                    "package $name {\n"
-                  . (' ' x $Sidef::SPACES)
-                  . "sub new {\n"
-                  . (' ' x $Sidef::SPACES)
-                  . $self->_dump_sub_init_vars('undef', @{$obj->{vars}})
-
-                  . (' ' x ($Sidef::SPACES * 2))
-                  . "bless {"
-                  . join(", ", map { $self->_dump_string($_->{name}) . " => " . $self->_dump_var($_) } @{$obj->{vars}})
-                  . "}, __PACKAGE__" . "\n"
-                  . (' ' x $Sidef::SPACES) . "}\n"
-                  .
-
-                  (' ' x $Sidef::SPACES) . "*call = \\&new;\n" .
-
-                  (' ' x $Sidef::SPACES)
-                  . join("\n" . (' ' x $Sidef::SPACES),
-                         map { "sub $_->{name} : lvalue { \$_[0]->{$_->{name}} }" } @{$obj->{vars}})
-
-                  . "\n" . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}";
-
-                $Sidef::SPACES -= $Sidef::SPACES_INCR;
-            }
-        }
-        elsif ($ref eq 'Sidef::Variable::LocalInit') {
-            $code = 'local $' . $obj->{class} . '::' . $obj->{name};
-        }
-        elsif ($ref eq 'Sidef::Variable::LocalMagic') {
-            $code = 'local ' . $obj->{name};
+        elsif ($ref eq 'Sidef::Object::Unary') {
+            ## OK
         }
         elsif ($ref eq 'Sidef::Variable::Local') {
-            $code = '$' . $obj->{class} . '::' . $obj->{name};
+            $code = 'local ' . $self->deparse_script($obj->{expr});
         }
         elsif ($ref eq 'Sidef::Variable::Global') {
             $code = '$' . $obj->{class} . '::' . $obj->{name},;
         }
-        elsif ($ref eq 'Sidef::Object::Unary') {
-            ## OK
-        }
         elsif ($ref eq 'Sidef::Variable::Define') {
             my $name  = $obj->{name} . $refaddr;
             my $value = '(' . 'main::' . $name . ')';
@@ -627,7 +576,6 @@ HEADER
                 $obj->{inited} = 1;
                 $self->top_add('use constant ' . $name . ' => ' . 'do {' . $self->deparse_script($obj->{expr}) . " };\n");
             }
-
             $code = $value;
         }
         elsif ($ref eq 'Sidef::Variable::Const') {
@@ -711,7 +659,7 @@ HEADER
         }
         elsif ($ref eq 'Sidef::Types::Block::CodeInit') {
             if ($addr{$refaddr}++) {
-                $code = 'Sidef::Types::Block::Code->new(__SUB__)';
+                $code = 'Sidef::Types::Block::Code->new(code => __SUB__)';
             }
             else {
                 if (%{$obj}) {
@@ -721,6 +669,8 @@ HEADER
                     my $is_function = exists($self->{function}) && $self->{function} == $refaddr;
                     my $is_class    = exists($self->{class})    && $self->{class} == $refaddr;
 
+                    local $self->{current_block} = $refaddr;
+
                     if ($is_class) {
                         $code = " {\n";
 
@@ -741,8 +691,7 @@ HEADER
                             $code .= (" " x $Sidef::SPACES) . 'sub new {' . "\n";
 
                             $Sidef::SPACES += $Sidef::SPACES_INCR;
-
-                            $code .= $self->_dump_sub_init_vars('undef', @{$self->{class_vars}});
+                            $code .= $self->_dump_sub_init_vars(extended => 1, vars => ['undef', @{$self->{class_vars}}]);
 
                             $code .= " " x $Sidef::SPACES;
                             $code .= 'my $self = bless {';
@@ -768,11 +717,9 @@ HEADER
                         $code = 'Sidef::Types::Block::Code->new(';
                     }
 
-                    if ($is_class) {
+                    if (not $is_class) {
 
-                    }
-                    else {
-                        $code .= "\n" . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "sub {\n";
+                        $code .= "\n" . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "code => sub {\n";
 
                         if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
                             my @vars = @{$obj->{init_vars}{vars}};
@@ -782,7 +729,7 @@ HEADER
                                 pop @vars;
                             }
 
-                            $code .= $self->_dump_sub_init_vars(@vars);
+                            $code .= $self->_dump_sub_init_vars(extended => $is_function, vars => \@vars);
 
                             if ($is_function) {
                                 $code .= (' ' x $Sidef::SPACES) . 'my @return;' . "\n";
@@ -801,6 +748,13 @@ HEADER
                         }
                     }
 
+                    # Localize variable declarations
+                    while (    exists($self->{block_declarations})
+                           and @{$self->{block_declarations}}
+                           and $self->{block_declarations}[-1][0] == $refaddr) {
+                        $code .= (' ' x $Sidef::SPACES) . pop(@{$self->{block_declarations}})->[1] . "\n";
+                    }
+
                     # Make the last statement to be the return value
                     if ($is_function && @statements) {
 
@@ -819,40 +773,130 @@ HEADER
                         (" " x $Sidef::SPACES)
                       . join(";\n" . (" " x $Sidef::SPACES), @statements)
                       . ($is_function ? (";\n" . (" " x $Sidef::SPACES) . "END$refaddr: \@return;\n") : '') . "\n"
-                      . (" " x ($Sidef::SPACES -= $Sidef::SPACES_INCR))
-                      . ($is_class ? '}' : '})');
+                      . (" " x ($Sidef::SPACES -= $Sidef::SPACES_INCR)) . '}';
+
+                    if (not $is_class) {
+                        if (exists $self->{parent_name}) {
+                            $code .= ','
+                              . join(',',
+                                     'type => ' . $self->_dump_string($self->{parent_name}[0]),
+                                     'name => ' . $self->_dump_string($self->{parent_name}[1]));
+                        }
+
+                        if (exists $self->{class_name}) {
+                            $code .= ',' . 'class => ' . $self->_dump_string($self->{class_name});
+                        }
+                        $code .= ')';
+                    }
                 }
                 else {
                     $code = 'Block';
                 }
             }
         }
-        elsif ($ref eq 'Sidef::Variable::Ref') {
+        elsif ($ref eq 'Sidef::Variable::Struct') {
+            my $name = $self->_dump_class_name($obj);
+            if ($addr{$refaddr}++) {
+                $code = $name;
+            }
+            else {
+                local $self->{parent_name} = ['struct initialization', $obj->{name}];
+
+                # Mark the variables all in use
+                foreach my $var (@{$obj->{vars}}) {
+                    $var->{in_use} = 1;
+                }
+
+                $Sidef::SPACES += $Sidef::SPACES_INCR;
+                $code =
+                    "package $name {\n"
+                  . (' ' x $Sidef::SPACES)
+                  . "sub new {\n"
+                  . (' ' x $Sidef::SPACES)
+                  . $self->_dump_sub_init_vars(extended => 1, vars => ['undef', @{$obj->{vars}}])
+
+                  . (' ' x ($Sidef::SPACES * 2))
+                  . "bless {"
+                  . join(", ", map { $self->_dump_string($_->{name}) . " => " . $self->_dump_var($_) } @{$obj->{vars}})
+                  . "}, __PACKAGE__" . "\n"
+                  . (' ' x $Sidef::SPACES) . "}\n"
+                  .
+
+                  (' ' x $Sidef::SPACES) . "*call = \\&new;\n" .
+
+                  (' ' x $Sidef::SPACES)
+                  . join("\n" . (' ' x $Sidef::SPACES),
+                         map { "sub $_->{name} : lvalue { \$_[0]->{$_->{name}} }" } @{$obj->{vars}})
+
+                  . "\n" . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}";
+
+                $Sidef::SPACES -= $Sidef::SPACES_INCR;
+            }
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Number') {
+            my $value = $obj->get_value;
+
+            if (ref($value)) {
+                if (ref($value) eq 'Math::BigRat') {
+                    $value = (q{Math::BigRat->new('} . join('/', $value->parts) . q{')});
+                }
+                else {
+                    $value = (q{'} . $value->bstr . q{'});
+                }
+            }
+
+            $code = $self->make_constant($ref, 'new', "Number$refaddr", $value);
+        }
+        elsif ($ref eq 'Sidef::Types::String::String') {
+            $code = $self->make_constant($ref, 'new', "String$refaddr", $self->_dump_string(${$obj}));
+        }
+        elsif ($ref eq 'Sidef::Types::Array::Array' or $ref eq 'Sidef::Types::Array::HCArray') {
+            $code = $self->_dump_array('Sidef::Types::Array::Array', $obj);
+        }
+        elsif ($ref eq 'Sidef::Types::Bool::Bool') {
+            $code = $self->make_constant($ref, 'new', ${$obj} ? ("true$refaddr", 1) : ("false$refaddr", 0));
+        }
+        elsif ($ref eq 'Sidef::Types::Regex::Regex') {
+            $code =
+              $self->make_constant($ref, 'new', "Regex$refaddr",
+                                   $self->_dump_string("$obj->{regex}"),
+                                   $obj->{global} ? '"g"' : ());
+        }
+        elsif ($ref eq 'Sidef::Types::Block::If' or $ref eq 'Sidef::Types::Block::While') {
             ## ok
         }
-        elsif ($ref eq 'Sidef::Sys::Sys') {
-            $code = $self->make_constant($ref, 'new', "Sys$refaddr");
+        elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
+            $code = '('
+              . $self->deparse_script($obj->{cond}) . '?'
+              . $self->deparse_block_expr($obj->{true}) . ':'
+              . $self->deparse_block_expr($obj->{false}) . ')';
         }
-        elsif ($ref eq 'Sidef::Parser') {
-            $code = $ref . '->new';
+        elsif ($ref eq 'Sidef::Variable::NamedParam') {
+            $code = $ref . '->new(' . $self->_dump_string($obj->[0]) . ',' . $self->deparse_args(@{$obj->[1]}) . ')';
         }
-        elsif ($ref eq 'Sidef') {
-            $code = $self->make_constant($ref, 'new', "Sidef$refaddr");
+        elsif ($ref eq 'Sidef::Types::Nil::Nil') {
+            $code = 'undef';
         }
-        elsif ($ref eq 'Sidef::Object::Object') {
-            $code = $self->make_constant($ref, 'new', "Object$refaddr");
+        elsif ($ref eq 'Sidef::Types::Hash::Hash') {
+            if (keys(%{$obj})) {
+                $code = $ref . '->new(' . join(
+                    ',',
+                    map {
+                        $self->_dump_string($_) . ' => '
+                          . (defined($obj->{$_}) ? $self->deparse_expr({self => $obj->{$_}}) : 'undef')
+                      } keys(%{$obj})
+                  )
+                  . ')';
+            }
+            else {
+                $code = $self->make_constant($ref, 'new', "Hash$refaddr");
+            }
         }
-        elsif ($ref eq 'Sidef::Variable::LazyMethod') {
-            $code = $ref . '->new';
+        elsif ($ref eq 'Sidef::Types::Block::Do') {
+            $code = 'do {' . join(';', $self->deparse_script($obj->{block}{code})) . '}';
         }
-        elsif ($ref eq 'Sidef::Types::Block::For') {
-            ## ok
-        }
-        elsif ($ref eq 'Sidef::Types::Block::If') {
-            ## ok
-        }
-        elsif ($ref eq 'Sidef::Types::Block::While') {
-            ## ok
+        elsif ($ref eq 'Sidef::Types::Block::Loop') {
+            $code = 'while(1) {' . join(';', $self->deparse_script($obj->{block}{code})) . '}';
         }
         elsif ($ref eq 'Sidef::Types::Block::Given') {
             $self->top_add(qq{use experimental 'smartmatch';\n});
@@ -873,21 +917,12 @@ HEADER
             my $raddr = refaddr($obj->{gather});
             $code = "do { push \@_$raddr," . $self->deparse_args($obj->{expr}) . "; \$_$raddr\[-1] }";
         }
+        elsif ($ref eq 'Sidef::Types::Block::For') {
+            ## ok
+        }
         elsif ($ref eq 'Sidef::Types::Block::Try') {
             $code = $ref . '->new';
         }
-        elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
-            $code = '('
-              . $self->deparse_script($obj->{cond}) . '?'
-              . $self->deparse_block_expr($obj->{true}) . ':'
-              . $self->deparse_block_expr($obj->{false}) . ')';
-        }
-        elsif ($ref eq 'Sidef::Module::OO') {
-            $code = $self->make_constant($ref, '__NEW__', "MOD_OO$refaddr", $self->_dump_string($obj->{module}));
-        }
-        elsif ($ref eq 'Sidef::Module::Func') {
-            $code = $self->make_constant($ref, '__NEW__', "MOD_F$refaddr", $self->_dump_string($obj->{module}));
-        }
         elsif ($ref eq 'Sidef::Types::Block::Break') {
             $code = 'last';
         }
@@ -935,9 +970,6 @@ HEADER
         elsif ($ref eq 'Sidef::Variable::Magic') {
             $code = $obj->{name};
         }
-        elsif ($ref eq 'Sidef::Types::Hash::Hash') {
-            $code = $self->make_constant($ref, 'new', "Hash$refaddr");
-        }
         elsif ($ref eq 'Sidef::Types::Glob::Socket') {
             $code = $self->make_constant($ref, 'new', "Socket$refaddr");
         }
@@ -952,6 +984,7 @@ HEADER
             local \$Sidef::DEPARSER->{top_program} = '';
             local \$Sidef::DEPARSER->{_has_constant} = 0;
             local \$Sidef::DEPARSER->{function_declarations} = [];
+            local \$Sidef::DEPARSER->{block_declarations} = [];
             \$Sidef::DEPARSER->deparse(
             do {
                 local \$Sidef::PARSER->{vars} = \$Sidef::EVALS{$refaddr}{vars};
@@ -969,38 +1002,24 @@ HEADER
             $code = $self->make_constant($ref, 'new', "Complex$refaddr", "'" . ${$obj}->Re . "'", "'" . ${$obj}->Im . "'");
         }
         elsif ($ref eq 'Sidef::Types::Array::Pair') {
-            $code = $ref . '->new';
-        }
-        elsif ($ref eq 'Sidef::Variable::NamedParam') {
-            $code = $ref . '->new(' . $self->_dump_string($obj->[0]) . ',' . $self->deparse_args(@{$obj->[1]}) . ')';
-        }
-        elsif ($ref eq 'Sidef::Types::Regex::Regex') {
-            $code =
-              $self->make_constant($ref, 'new', "Regex$refaddr",
-                                   $self->_dump_string("$obj->{regex}"),
-                                   $obj->{global} ? '"g"' : ());
-        }
-        elsif ($ref eq 'Sidef::Types::Number::Number') {
-            my $value = $obj->get_value;
-            $code = $self->make_constant($ref, 'new', "Number$refaddr", ref($value) ? (q{'} . $value->bstr . q{'}) : $value);
-        }
-        elsif ($ref eq 'Sidef::Types::Array::Array' or $ref eq 'Sidef::Types::Array::HCArray') {
-            $code = $self->_dump_array('Sidef::Types::Array::Array', $obj);
-        }
-        elsif ($ref eq 'Sidef::Types::Nil::Nil') {
-            $code = 'undef';
+            if (all { not defined($_) } @{$obj}) {
+                $code = $self->make_constant($ref, 'new', "Pair$refaddr");
+            }
+            else {
+                $code =
+                    $ref
+                  . '->new('
+                  . join(', ', map { defined($_) ? $self->deparse_expr({self => $_}) : 'undef' } @{$obj}) . ')';
+            }
         }
         elsif ($ref eq 'Sidef::Types::Null::Null') {
             $code = $self->make_constant($ref, 'new', "Null$refaddr");
         }
-        elsif ($ref eq 'Sidef::Types::String::String') {
-            $code = $self->make_constant($ref, 'new', "String$refaddr", $self->_dump_string(${$obj}));
+        elsif ($ref eq 'Sidef::Module::OO') {
+            $code = $self->make_constant($ref, '__NEW__', "MOD_OO$refaddr", $self->_dump_string($obj->{module}));
         }
-        elsif ($ref eq 'Sidef::Types::Bool::Bool') {
-            $code = $self->make_constant($ref, 'new', ${$obj} ? ("true$refaddr", 1) : ("false$refaddr", 0));
-        }
-        elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
-            $code = $ref . '->new';
+        elsif ($ref eq 'Sidef::Module::Func') {
+            $code = $self->make_constant($ref, '__NEW__', "MOD_F$refaddr", $self->_dump_string($obj->{module}));
         }
         elsif ($ref eq 'Sidef::Types::Range::RangeNumber' or $ref eq 'Sidef::Types::Range::RangeString') {
             $code = $ref . '->new';
@@ -1014,6 +1033,57 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Glob::Dir') {
             $code = $self->make_constant($ref, 'new', "Dir$refaddr", $self->_dump_string(${$obj}));
         }
+        elsif ($ref eq 'Sidef::Sys::Sys') {
+            $code = $self->make_constant($ref, 'new', "Sys$refaddr");
+        }
+        elsif ($ref eq 'Sidef::Meta::Assert') {
+            my @args = $self->deparse_script($obj->{arg});
+
+            if ($obj->{act} eq 'assert') {
+
+                # Check arity
+                @args == 1
+                  or die "[ERROR] Incorrect number of arguments for $obj->{act}\() at"
+                  . " $obj->{file} line $obj->{line} (expected 1 argument)\n";
+
+                # Generate code
+                $code = qq~do{do{$args[0]} or CORE::die "$obj->{act}\Q$obj->{code}\E failed ~
+                  . qq~in \Q$obj->{file}\E at line $obj->{line}\\n"}~;
+            }
+            elsif ($obj->{act} eq 'assert_eq' or $obj->{act} eq 'assert_ne') {
+
+                # Check arity
+                @args == 2
+                  or die "[ERROR] Incorrect number of arguments for $obj->{act}\() at"
+                  . " $obj->{file} line $obj->{line} (expected 2 arguments)\n";
+
+                # Generate code
+                $code = "do{"
+                  . ($obj->{act} eq 'assert_ne' ? qq{not(do{$args[0]} eq do{$args[1]})} : qq{do{$args[0]} eq do{$args[1]}})
+                  . qq{ or CORE::die "$obj->{act}\Q$obj->{code}\E failed at \Q$obj->{file}\E line $obj->{line}\\n\"\}};
+            }
+        }
+        elsif ($ref eq 'Sidef::Meta::Error') {
+            my @args = $self->deparse_args($obj->{arg});
+            $code = qq~do{CORE::die(@args, " at \Q$obj->{file}\E line $obj->{line}\\n")}~;
+        }
+        elsif ($ref eq 'Sidef::Meta::Warning') {
+            my @args = $self->deparse_args($obj->{arg});
+            $code = qq~Sidef::Types::Bool::Bool->new(CORE::warn(@args, " at \Q$obj->{file}\E line $obj->{line}\\n"))~;
+        }
+        elsif ($ref eq 'Sidef::Object::Object') {
+            $code = $self->make_constant($ref, 'new', "Object$refaddr");
+        }
+        elsif ($ref eq 'Sidef::Variable::LazyMethod') {
+            $code = $ref . '->new';
+        }
+        elsif ($ref eq 'Sidef::Types::Block::ForArray') {
+            $code =
+                'for my '
+              . $self->deparse_expr({self => $obj->{var}}) . '(@{'
+              . $self->deparse_expr({self => $obj->{array}}) . '})'
+              . $self->deparse_bare_block($obj->{block}->{code});
+        }
         elsif ($ref eq 'Sidef::Types::Byte::Bytes') {
             $code = $self->_dump_array($ref, $obj);
         }
@@ -1032,9 +1102,18 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Grapheme::Grapheme') {
             $code = $self->make_constant($ref, 'new', "Grapheme$refaddr", $self->_dump_string(${$obj}));
         }
+        elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
+            $code = $ref . '->new';
+        }
         elsif ($ref eq 'Sidef::Types::Glob::Pipe') {
             $code = $self->make_constant($ref, 'new', "Pipe$refaddr", map { $self->_dump_string($_) } @{$obj});
         }
+        elsif ($ref eq 'Sidef::Parser') {
+            $code = $ref . '->new';
+        }
+        elsif ($ref eq 'Sidef') {
+            $code = $self->make_constant($ref, 'new', "Sidef$refaddr");
+        }
 
         # Array indices
         if (exists $expr->{ind}) {
@@ -1146,18 +1225,6 @@ HEADER
                         }
                     }
 
-                    # Do-block
-                    if ($code eq '' and $ref eq 'Sidef::Types::Block::Do') {
-                        my $arg = $self->deparse_generic('', ';', '', @{$call->{arg}});
-
-                        if ($arg =~ s/^Sidef::Types::Block::Code->new\(\s*sub\h*\{//) {
-                            $arg =~ s/\}\)\z//;
-                        }
-
-                        $code = 'do { ' . $arg . '}';
-                        next;
-                    }
-
                     # Postfix ++ and -- operators on variables
                     if (exists($self->{inc_dec_ops}{$method})) {
                         $code = "do{my \$old=$code; $code=$code\->$self->{inc_dec_ops}{$method}; \$old}";
@@ -1250,30 +1317,8 @@ HEADER
                     }
                     elsif ($method =~ /^[\pL_]/) {
 
-                        # Optimize the "loop {}" construct
-                        if ($ref eq 'Sidef::Types::Block::CodeInit' and $method eq 'loop') {
-                            my $block = $code =~ s/^\(Sidef::Types::Block::Code->new\(\s*sub\h*(?=\{)//r =~ s/\)\)\z//r;
-
-                            if (not defined $block) {
-                                die "[ERROR] Failed to optimize the 'loop {}' construct...";
-                            }
-
-                            $code = 'while (1) ' . $block;
-                            next;
-                        }
-
-                        # Optimize the "n.times {}" construct
-                        elsif ($ref eq 'Sidef::Types::Number::Number' and $method eq 'times' and $$obj < (-1 >> 1)) {
-
-                            my $arg   = $self->deparse_args(@{$call->{arg}});
-                            my $block = $arg =~ s/^\(Sidef::Types::Block::Code->new\(\s*sub\h*\{//r =~ s/\)\)\z//r;
-
-                            $code = "for (1..$$obj) { local \@_ = (Sidef::Types::Number::Number->new(\$_));  " . $block;
-                            next;
-                        }
-
                         # Exclamation mark (!) at the end of a method
-                        elsif (substr($method, -1) eq '!') {
+                        if (substr($method, -1) eq '!') {
                             $code = '('
                               . "$old_code=$code->"
                               . substr($method, 0, -1)
@@ -1383,6 +1428,11 @@ HEADER
               ? ("\n" . join("\n", map { $_->[1] } @{$self->{function_declarations}}) . "\n")
               : ''
              )
+           . (
+              exists($self->{block_declarations})
+                && @{$self->{block_declarations}} ? ("\n" . join("\n", map { $_->[1] } @{$self->{block_declarations}}) . "\n")
+              : ''
+             )
            . $self->{top_program} . "\n"
            . join($self->{between}, @statements)
            . $self->{after}
