@@ -10,17 +10,12 @@ package Sidef::Deparse::Sidef {
         my (undef, %args) = @_;
 
         my %opts = (
-                    before         => '',
-                    between        => ";\n",
-                    after          => ";\n",
-                    class          => 'main',
-                    extra_parens   => 0,
-                    namespaces     => [],
-                    obj_with_block => {
-                                       'Sidef::Types::Block::While' => {
-                                                                        while => 1,
-                                                                       },
-                                      },
+                    before       => '',
+                    between      => ";\n",
+                    after        => ";\n",
+                    class        => 'main',
+                    extra_parens => 0,
+                    namespaces   => [],
                     %args,
                    );
         %addr = ();    # reset the addr map
@@ -107,22 +102,29 @@ package Sidef::Deparse::Sidef {
     sub _dump_string {
         my ($self, $str) = @_;
 
+        return 'String' if $str eq '';
         state $x = eval { require Data::Dump };
         $x || return ('"' . quotemeta($str) . '"');
 
         Data::Dump::quote($str);
     }
 
+    sub _dump_number {
+        my ($self, $num) = @_;
+
+        state $table = {
+                        'inf'  => q{Number.inf},
+                        '-inf' => q{Number.inf('-')},
+                        'nan'  => q{Number.nan},
+                        '0'    => q{Number},
+                       };
+
+        exists($table->{lc($num)}) ? $table->{lc($num)} : $num;
+    }
+
     sub _dump_array {
         my ($self, $array) = @_;
-        '[' . join(
-            ', ',
-
-            ref($array) eq 'Sidef::Types::Array::Array'
-            ? (map { $self->deparse_expr({self => $_->get_value}) } @{$array})
-            : (map { $self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}) } @{$array})
-          )
-          . ']';
+        '[' . join(', ', map { $self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}) } @{$array}) . ']';
     }
 
     sub _dump_class_name {
@@ -176,16 +178,12 @@ package Sidef::Deparse::Sidef {
                       . $self->_dump_vars(@{$var_obj->{vars}}[($obj->{type} eq 'method' ? 1 : 0) .. $#{$var_obj->{vars}} - 1])
                       . ') ';
 
-                   #$code .= $self->_dump_sub_init_vars($vars);
-                   #$code .= $self->_dump_init_vars($vars);
-                   #$code .= '(' . $self->_dump_init_vars(@{$vars}[($obj->{type} eq 'method' ? 1 : 0) .. $#{$vars} - 1]) . ')';
-
                     if (exists $obj->{cached}) {
                         $code .= 'is cached ';
                     }
 
                     if (exists $obj->{returns}) {
-                        $code .= '-> ' . $self->deparse_expr({self => $obj->{returns}}) . ' ';
+                        $code .= '-> (' . join(',', map { $self->deparse_expr({self => $_}) } @{$obj->{returns}}) . ') ';
                     }
 
                     $code .= $self->deparse_expr({self => $block});
@@ -206,11 +204,8 @@ package Sidef::Deparse::Sidef {
                 $code = "struct $obj->{name} {" . $self->_dump_vars(@{$obj->{vars}}) . '}';
             }
         }
-        elsif ($ref eq 'Sidef::Variable::LocalInit') {
-            $code = "local $obj->{name}";
-        }
         elsif ($ref eq 'Sidef::Variable::Local') {
-            $code = "$obj->{name}";
+            $code = 'local ' . '(' . $self->deparse_script($obj->{expr}) . ')';
         }
         elsif ($ref eq 'Sidef::Variable::Global') {
             $code = 'global ' . $obj->{class} . '::' . $obj->{name},;
@@ -221,12 +216,6 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Variable::ConstInit') {
             $code = join(";\n" . (" " x $Sidef::SPACES), map { $self->deparse_expr({self => $_}) } @{$obj->{vars}});
         }
-        elsif ($ref eq 'Sidef::Variable::LocalInit') {
-            $code = 'local ' . $obj->{class} . '::' . $obj->{name};
-        }
-        elsif ($ref eq 'Sidef::Variable::LocalMagic') {
-            $code = 'local ' . $obj->{name};
-        }
         elsif ($ref eq 'Sidef::Types::Range::RangeNumber' or $ref eq 'Sidef::Types::Range::RangeString') {
             $code = $self->_dump_reftype($obj);
         }
@@ -305,7 +294,7 @@ package Sidef::Deparse::Sidef {
                         my @vars = @{$obj->{init_vars}{vars}};
                         pop @vars;
                         if (@vars) {
-                            $code .= '|' . $self->_dump_vars(@vars) . "|";
+                            $code .= '| ' . $self->_dump_vars(@vars) . ' |';
                         }
                     }
 
@@ -339,8 +328,17 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Sys::Sys') {
             $code = exists($obj->{file_name}) ? '' : 'Sys';
         }
+        elsif ($ref eq 'Sidef::Meta::Assert') {
+            $code = $obj->{act} . $self->deparse_args($obj->{arg});
+        }
+        elsif ($ref eq 'Sidef::Meta::Error') {
+            $code = 'die' . $self->deparse_args($obj->{arg});
+        }
+        elsif ($ref eq 'Sidef::Meta::Warning') {
+            $code = 'warn' . $self->deparse_args($obj->{arg});
+        }
         elsif ($ref eq 'Sidef::Eval::Eval') {
-            $code = 'eval(' . $self->deparse_script($obj->{expr}) . ')';
+            $code = 'eval' . $self->deparse_args($obj->{expr});
         }
         elsif ($ref eq 'Sidef::Parser') {
             $code = 'Parser';
@@ -358,17 +356,13 @@ package Sidef::Deparse::Sidef {
             $code = 'LazyMethod';
         }
         elsif ($ref eq 'Sidef::Types::Block::Break') {
-            if (not exists $expr->{call}) {
-                $code = 'break';
-            }
+            $code = 'break';
         }
         elsif ($ref eq 'Sidef::Types::Block::Default') {
             $code = 'default' . $self->deparse_bare_block($obj->{block}->{code});
         }
         elsif ($ref eq 'Sidef::Types::Block::Next') {
-            if (not exists $expr->{call}) {
-                $code = 'next';
-            }
+            $code = 'next';
         }
         elsif ($ref eq 'Sidef::Types::Block::Continue') {
             $code = 'continue';
@@ -399,6 +393,18 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Types::Block::Take') {
             $code = 'take' . $self->deparse_args($obj->{expr});
         }
+        elsif ($ref eq 'Sidef::Types::Block::Do') {
+            $code = 'do ' . $self->deparse_expr({self => $obj->{block}});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::Loop') {
+            $code = 'loop ' . $self->deparse_expr({self => $obj->{block}});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::ForArray') {
+            $code = 'for '
+              . $self->deparse_expr({self => $obj->{var}}) . ' in ('
+              . $self->deparse_expr({self => $obj->{array}}) . ') '
+              . $self->deparse_bare_block($obj->{block}->{code});
+        }
         elsif ($ref eq 'Sidef::Math::Math') {
             $code = 'Math';
         }
@@ -453,13 +459,18 @@ package Sidef::Deparse::Sidef {
         }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
             my $value = $obj->get_value;
-            my $num = ref($value) ? ref($value) eq 'Math::BigRat' ? $value->numify : $value->bstr : $value;
-            $code =
-                $num eq '0'        ? 'Number'
-              : lc($num) eq 'inf'  ? '0.inf'
-              : lc($num) eq '-inf' ? "0.inf('-')"
-              : lc($num) eq 'nan'  ? '0.nan'
-              :                      $num;
+
+            if (ref($value)) {
+                if (ref($value) eq 'Math::BigRat') {
+                    $code = '(' . join('//', map { $self->_dump_number($_) } $value->parts()) . ')';
+                }
+                else {
+                    $code = $self->_dump_number($value->bstr);
+                }
+            }
+            else {
+                $code = $self->_dump_number($value);
+            }
         }
         elsif ($ref eq 'Sidef::Types::Array::Array' or $ref eq 'Sidef::Types::Array::HCArray') {
             if (not @{$obj}) {
