@@ -1,7 +1,7 @@
 package Sidef::Optimizer {
 
     use 5.014;
-    require Scalar::Util;
+    use Scalar::Util qw(refaddr);
 
     use constant {
                   STRING  => 'Sidef::Types::String::String',
@@ -22,7 +22,13 @@ package Sidef::Optimizer {
             my ($package, @names) = @_;
             my $module = $cache{$package} //= (($package =~ s{::}{/}gr) . '.pm');
             exists($INC{$module}) || require($module);
-            map { \&{$package . '::' . $_} } @names;
+            map {
+
+                # defined(&{$package . '::' . $_})
+                #   or warn "Invalid method $package: $_";
+
+                \&{$package . '::' . $_}
+            } @names;
         }
     }
 
@@ -38,14 +44,18 @@ package Sidef::Optimizer {
              map {
                  { $_, [table(STRING)] }
                } methods(STRING, qw(
+                   new call
+
                    concat
+                   prepend
+
                    gt lt le ge cmp
 
                    xor and or
                    eq ne
 
                    index
-                   unpack
+                   crypt
 
                    levenshtein
                    contains
@@ -68,11 +78,21 @@ package Sidef::Optimizer {
              map {
                  { $_, [] }
                } methods(STRING, qw(
-                   lc uc tc wc tclc lcfirst
+                   lc uc fc tc wc tclc lcfirst
 
                    pop
+                   chop
                    chomp
-                   length
+
+                   chars_len
+                   bytes_len
+                   graphs_len
+
+                   chars
+                   bytes
+                   lines
+                   words
+                   graphemes
 
                    ord
                    oct
@@ -81,10 +101,16 @@ package Sidef::Optimizer {
                    num
                    not
 
+                   first
+                   last
+
                    repeat
                    reverse
                    clear
+                   sort
+
                    is_empty
+                   is_palindrome
 
                    trim
                    trim_beg
@@ -114,7 +140,7 @@ package Sidef::Optimizer {
 
                    times
                    repeat
-                   char_at
+                   char
 
                    sprintf
                    sprintlnf
@@ -126,7 +152,11 @@ package Sidef::Optimizer {
             ),
 
             # String.method(String | Number | Regex)
-            #(map {{$_, [table(STRING, NUMBER, REGEX)]}} methods(STRING, qw(split))),
+            (
+             map {
+                 { $_, [table(STRING, NUMBER, REGEX)] }
+               } methods(STRING, qw(split))
+            ),
 
             # String.method(String, String)
             (
@@ -170,25 +200,27 @@ package Sidef::Optimizer {
              map {
                  { $_, [table(NUMBER)] }
                } methods(NUMBER, qw(
+                   new call
+
                    + - / * % **
 
                    lt gt le ge cmp acmp
                    eq ne
                    and or xor
 
-                   divmod
-
+                   digit
                    complex
                    root log
+                   npow
                    max min
                    round roundf
                    digit
                    nok
+                   rdiv
                    is_div
 
                    shift_right
                    shift_left
-                   next_power_of
                    )
                )
             ),
@@ -198,10 +230,13 @@ package Sidef::Optimizer {
              map {
                  { $_, [] }
                } methods(NUMBER, qw(
+                   new call
                    inc dec not
 
                    factorial
                    sqrt
+                   npow2
+                   troot
                    abs
 
                    hex oct bin
@@ -209,13 +244,14 @@ package Sidef::Optimizer {
                    cos sin
                    log ln log10 log2
 
-                   infinity
-                   negate
+                   inf
+                   neg
                    sign
                    nan
                    chr
 
                    is_zero
+                   is_one
                    is_nan
                    is_positive
                    is_negative
@@ -228,16 +264,21 @@ package Sidef::Optimizer {
                    floor
                    length
 
+                   numerator
+                   denominator
+
+                   digits
+
                    as_bin
                    as_oct
                    as_hex
 
+                   rat
                    complex i
 
                    sstr
                    dump
                    commify
-                   next_power_of_two
                    )
                )
             ),
@@ -247,6 +288,7 @@ package Sidef::Optimizer {
              map {
                  { $_, [table(NUMBER), table(NUMBER)] }
                } methods(NUMBER, qw(
+                   modpow
                    shift_right
                    shift_left
                    )
@@ -280,15 +322,19 @@ package Sidef::Optimizer {
 
                    len end
                    is_empty
-
                    min max
-                   minmax
 
                    sum
                    prod
 
+                   sort
+                   reverse
+
+                   unique
+                   last_unique
+                   flatten
+
                    to_s
-                   to_list
                    dump
                    )
                )
@@ -298,8 +344,47 @@ package Sidef::Optimizer {
              map {
                  { $_, [table(NUMBER)] }
                } methods(ARRAY, qw(
+                   count
+                   index
+                   rindex
                    exists
                    defined
+                   contains
+                   contains_type
+
+                   divide
+                   multiply
+
+                   rotate
+
+                   first
+                   last
+                   item
+                   ft
+
+                   sum
+                   prod
+
+                   take_right
+                   take_left
+                   )
+               )
+            ),
+
+            (
+             map {
+                 { $_, [table(ARRAY)] }
+               } methods(ARRAY, qw(
+                   and
+                   or
+                   xor
+                   concat
+
+                   eq ne
+                   mzip
+                   contains_type
+                   contains_any
+                   contains_all
                    )
                )
             ),
@@ -310,6 +395,13 @@ package Sidef::Optimizer {
                } methods(ARRAY, qw(
                    pack
                    join
+                   index
+                   rindex
+                   count
+                   contains
+                   contains_type
+                   reduce
+                   reduce_operator
                    )
                )
             ),
@@ -414,13 +506,45 @@ package Sidef::Optimizer {
 
         (COMPLEX) => [
 
-            # Complex.method(Complex)
+            # Complex.method(Complex|Number)
             (
              map {
-                 { $_, [table(COMPLEX)] }
+                 { $_, [table(COMPLEX, NUMBER)] }
                } methods(COMPLEX, qw(
-                   cmp
+                   cmp gt lt ge le eq ne
                    roundf
+
+                   mul
+                   div
+                   add
+                   sub
+                   exp
+                   log
+                   pow
+
+                   atan2
+                   )
+               )
+            ),
+
+            # Complex.method(Number|Complex, Number|Complex)
+            (
+             map {
+                 { $_, [table(NUMBER, COMPLEX)] }
+               } methods(COMPLEX, qw(
+                   call
+                   new
+                   )
+               )
+            ),
+
+            # Complex.method(Number|Complex, Number|Complex)
+            (
+             map {
+                 { $_, [table(NUMBER, COMPLEX), table(NUMBER, COMPLEX)] }
+               } methods(COMPLEX, qw(
+                   call
+                   new
                    )
                )
             ),
@@ -430,11 +554,45 @@ package Sidef::Optimizer {
              map {
                  { $_, [] }
                } methods(COMPLEX, qw(
+                   new call
+
                    inc
                    dec
+                   abs
+
+                   log
+                   log10
+                   sqrt
+
+                   cos
+                   sin
+                   tan
+                   csc
+                   sec
+                   cot
+                   asin
+                   acos
+                   atan
+                   acsc
+                   asec
+                   acot
+                   sinh
+                   cosh
+                   tanh
+                   csch
+                   sech
+                   coth
+                   asinh
+                   acosh
+                   atanh
+                   acsch
+                   asech
+                   acoth
+
+                   pi
 
                    int
-                   negate
+                   neg
                    not
                    sign
 
@@ -442,9 +600,10 @@ package Sidef::Optimizer {
                    imaginary
 
                    is_zero
+                   is_one
                    is_nan
-                   is_positive
-                   is_negative
+                   is_pos
+                   is_neg
                    is_even
                    is_odd
                    is_inf
@@ -492,7 +651,7 @@ package Sidef::Optimizer {
                 ## ok
             }
             elsif ($obj->{type} eq 'func' or $obj->{type} eq 'method') {
-                if ($addr{Scalar::Util::refaddr($obj)}++) {
+                if ($addr{refaddr($obj)}++) {
                     ## ok
                 }
                 else {
@@ -501,15 +660,24 @@ package Sidef::Optimizer {
             }
         }
         elsif ($ref eq 'Sidef::Variable::ClassInit') {
-            if ($addr{Scalar::Util::refaddr($obj)}++) {
+            if ($addr{refaddr($obj)}++) {
                 ## ok
             }
             else {
                 $obj->{block} = $self->optimize_expr({self => $obj->{block}});
             }
         }
+        elsif ($ref eq 'Sidef::Variable::Static') {
+            if ($addr{refaddr($obj)}++) {
+                ## ok
+            }
+            else {
+                my %code = $self->optimize($obj->{expr});
+                $obj->{expr} = \%code;
+            }
+        }
         elsif ($ref eq 'Sidef::Variable::Init') {
-            if ($addr{Scalar::Util::refaddr($obj)}++) {
+            if ($addr{refaddr($obj)}++) {
                 ## ok
             }
             else {
@@ -519,8 +687,35 @@ package Sidef::Optimizer {
                 }
             }
         }
+        elsif ($ref eq 'Sidef::Types::Block::Do') {
+            if ($addr{refaddr($obj)}++) {
+                ## ok
+            }
+            else {
+                my %code = $self->optimize($obj->{block}{code});
+                $obj->{block}{code} = \%code;
+            }
+        }
+        elsif ($ref eq 'Sidef::Types::Block::ForArray') {
+            if ($addr{refaddr($obj)}++) {
+                ## ok
+            }
+            else {
+                my %code = $self->optimize($obj->{block}{code});
+                $obj->{block}{code} = \%code;
+            }
+        }
+        elsif ($ref eq 'Sidef::Types::Block::Loop') {
+            if ($addr{refaddr($obj)}++) {
+                ## ok
+            }
+            else {
+                my %code = $self->optimize($obj->{block}{code});
+                $obj->{block}{code} = \%code;
+            }
+        }
         elsif ($ref eq 'Sidef::Types::Block::CodeInit') {
-            if ($addr{Scalar::Util::refaddr($obj)}++) {
+            if ($addr{refaddr($obj)}++) {
                 ## ok
             }
             else {
@@ -529,11 +724,19 @@ package Sidef::Optimizer {
             }
         }
         elsif ($ref eq 'Sidef::Types::Array::HCArray') {
+            my $has_expr = 0;
+
             foreach my $i (0 .. $#{$obj}) {
                 if (ref($obj->[$i]) eq 'HASH') {
                     $obj->[$i] = $self->optimize_expr($obj->[$i]);
+                    $has_expr ||= ref($obj->[$i]) eq 'HASH';
                 }
             }
+
+            # Has no expressions, so let's convert it into an Array
+            if (not $has_expr) {
+                $obj = Sidef::Types::Array::Array->new(@{$obj});
+            }
         }
 
         if (not exists($expr->{ind}) and not exists($expr->{lookup}) and not exists($expr->{call})) {
@@ -698,51 +901,3 @@ package Sidef::Optimizer {
 };
 
 1;
-
-__END__
-use utf8;
-use 5.014;
-use strict;
-use warnings;
-
-# The directory where Sidef lives
-use lib qw(..);
-
-# Load the Sidef main module
-use Sidef;
-
-#$SIG{__WARN__} = sub {die @_};
-
-# Initialize a new parser
-my $parser = Sidef::Parser->new();
-
-# Parse some code and store the returned parse-tree
-my $struct = $parser->parse_script(code => \<<'SIDEF_CODE');
-
-say "a"+("b"+"c")+"d".uc
-#split('', "test");
-#say "test".split('').map{.uc}.join;
-#say ("a" + "b" + "c" -> + "z")
-
-#say "x"+"y"
-#var x = "hei";
-#say ("test" + "z" + "d");
-
-#var x = "hei";
-#say ("a" + x + "b" + "c");
-
-#say "trizentrizen".index("r", 3);
-#say true.not.dump;
-
-#say ["a", "b" + "c"].len;
-
-#say %w(a b c).len;
-
-SIDEF_CODE
-
-use Data::Dump qw(pp);
-my $opt        = Sidef::Optimizer->new;
-my %opt_struct = $opt->optimize($struct);
-pp \%opt_struct;
-
-Sidef::Types::Block::Code->new(\%opt_struct)->run;
