@@ -6,8 +6,7 @@ package Sidef::Sys::Sys {
       );
 
     sub new {
-        my (undef, %opt) = @_;
-        CORE::bless \%opt, __PACKAGE__;
+        CORE::bless {}, __PACKAGE__;
     }
 
     sub exit {
@@ -109,39 +108,14 @@ package Sidef::Sys::Sys {
 
     sub die {
         my ($self, @args) = @_;
-        if (exists $self->{file_name}) {
-            CORE::die(@args, " at $self->{file_name} line $self->{line}.\n");
-        }
-        else {
-            CORE::die(@args, "\n");
-        }
+        CORE::die(@args, "\n");
     }
 
     *raise = \&die;
 
-    sub assert {
-        my ($self, $bool) = @_;
-        $bool || $self->die("assert: not true");
-    }
-
-    sub assert_eq {
-        my ($self, $arg1, $arg2) = @_;
-        Sidef::Types::Bool::Bool->new(($arg1 eq $arg2) || $self->die("assert_eq: $arg1 == $arg2 is false"));
-    }
-
-    sub assert_ne {
-        my ($self, $arg1, $arg2) = @_;
-        Sidef::Types::Bool::Bool->new(not($arg1 eq $arg2) || $self->die("assert_eq: $arg1 != $arg2 is false"));
-    }
-
     sub warn {
         my ($self, @args) = @_;
-        if (exists $self->{file_name}) {
-            CORE::warn(@args, " at $self->{file_name} line $self->{line}.\n");
-        }
-        else {
-            CORE::warn(@args, "\n");
-        }
+        CORE::warn(@args, "\n");
     }
 
     sub print {
@@ -184,23 +158,19 @@ package Sidef::Sys::Sys {
     *readln = \&scanln;
 
     sub read {
-        my ($self, $type, @vars) = @_;
+        my ($self, $type, $opt_arg) = @_;
 
-        if (@vars) {
-            foreach my $var_ref (@vars) {
-                chomp(my $input = <STDIN>);
-                ${$var_ref} = $type->new($input);
-            }
-
-            return $self;
+        if (defined $opt_arg) {
+            print $type;
+            $type = $opt_arg;
         }
 
         if (defined $type) {
-            chomp(my $input = <STDIN>);
+            chomp(my $input = <STDIN> // return);
             return $type->new($input);
         }
 
-        chomp(my $input = <STDIN>);
+        chomp(my $input = <STDIN> // return);
         Sidef::Types::String::String->new($input);
     }
 
