@@ -38,9 +38,7 @@ package Sidef::Types::Array::Array {
     sub unroll_operator {
         my ($self, $operator, $arg) = @_;
 
-        if (ref $operator) {
-            $operator = $operator->get_value;
-        }
+        $operator = $operator->get_value if ref($operator);
 
         my @array;
         if (defined $arg) {
@@ -63,9 +61,7 @@ package Sidef::Types::Array::Array {
     sub map_operator {
         my ($self, $operator, @args) = @_;
 
-        if (ref $operator) {
-            $operator = $operator->get_value;
-        }
+        $operator = $operator->get_value if ref($operator);
 
         my @array;
         foreach my $i (0 .. $#{$self}) {
@@ -78,9 +74,7 @@ package Sidef::Types::Array::Array {
     sub pam_operator {
         my ($self, $operator, $arg) = @_;
 
-        if (ref $operator) {
-            $operator = $operator->get_value;
-        }
+        $operator = $operator->get_value if ref($operator);
 
         my @array;
         foreach my $i (0 .. $#{$self}) {
@@ -93,19 +87,64 @@ package Sidef::Types::Array::Array {
     sub reduce_operator {
         my ($self, $operator) = @_;
 
-        if (ref $operator) {
-            $operator = $operator->get_value;
-        }
-
+        $operator = $operator->get_value if ref($operator);
         (my $offset = $#{$self}) >= 0 || return;
 
         my $x = $self->[0];
         foreach my $i (1 .. $offset) {
-            $x = ($x->$operator($self->[$i]));
+            $x = $x->$operator($self->[$i]);
         }
         $x;
     }
 
+    sub cross_operator {
+        my ($self, $operator, $arg) = @_;
+
+        $operator = $operator->get_value if ref($operator);
+
+        my @array;
+        if ($operator eq '') {
+            foreach my $i (@{$self}) {
+                foreach my $j (@{$arg}) {
+                    push @array, $self->new($i, $j);
+                }
+            }
+        }
+        else {
+            foreach my $i (@{$self}) {
+                foreach my $j (@{$arg}) {
+                    push @array, $i->$operator($j);
+                }
+            }
+        }
+
+        $self->new(@array);
+    }
+
+    sub zip_operator {
+        my ($self, $operator, $arg) = @_;
+
+        $operator = $operator->get_value if ref($operator);
+
+        my $self_len = $#{$self};
+        my $arg_len  = $#{$arg};
+        my $min      = $self_len < $arg_len ? $self_len : $arg_len;
+
+        my @array;
+        if ($operator eq '') {
+            foreach my $i (0 .. $min) {
+                push @array, $self->new($self->[$i], $arg->[$i]);
+            }
+        }
+        else {
+            foreach my $i (0 .. $min) {
+                push @array, $self->[$i]->$operator($arg->[$i]);
+            }
+        }
+
+        $self->new(@array);
+    }
+
     sub _grep {
         my ($self, $array, $bool) = @_;
 
@@ -184,6 +223,8 @@ package Sidef::Types::Array::Array {
         $self->_grep($array, 1);
     }
 
+    *sub = \&subtract;
+
     sub concat {
         my ($self, $arg) = @_;
 
@@ -228,7 +269,7 @@ package Sidef::Types::Array::Array {
         my $head = shift @set;
         my @result = _combinations($n - 1, @set);
         foreach my $subarray (@result) {
-            unshift @{$subarray}, $head;
+            CORE::unshift @{$subarray}, $head;
         }
         @result, _combinations($n, @set);
     }
@@ -348,13 +389,31 @@ package Sidef::Types::Array::Array {
     }
 
     sub sum {
-        $_[0]->reduce_operator('+');
+        defined($_[1])
+          ? do {
+            my $sum = $_[1];
+            state $method = '+';
+            foreach my $obj (@{$_[0]}) {
+                $sum = $sum->$method($obj);
+            }
+            $sum;
+          }
+          : $_[0]->reduce_operator('+');
     }
 
     *collapse = \&sum;
 
     sub prod {
-        $_[0]->reduce_operator('*');
+        defined($_[1])
+          ? do {
+            my $prod = $_[1];
+            state $method = '*';
+            foreach my $obj (@{$_[0]}) {
+                $prod = $prod->$method($obj);
+            }
+            $prod;
+          }
+          : $_[0]->reduce_operator('*');
     }
 
     *product = \&prod;
@@ -467,6 +526,11 @@ package Sidef::Types::Array::Array {
         exists($self->[$index]) ? $self->[$index] : ();
     }
 
+    sub fetch {
+        my ($self, $index, $default) = @_;
+        exists($self->[$index]) ? $self->[$index] : $default;
+    }
+
     sub _slice {
         my ($self, $from, $to) = @_;
 
@@ -555,6 +619,19 @@ package Sidef::Types::Array::Array {
 
     *collect = \&map;
 
+    sub map_with_index {
+        my ($self, $code) = @_;
+
+        my @arr;
+        foreach my $i (0 .. $#{$self}) {
+            push @arr, $code->run(Sidef::Types::Number::Number->new($i), $self->[$i]);
+        }
+
+        $self->new(@arr);
+    }
+
+    *collect_with_index = \&map_with_index;
+
     sub flat_map {
         my ($self, $code) = @_;
         $self->new(map { @{scalar $code->run($_)} } @{$self});
@@ -644,10 +721,9 @@ package Sidef::Types::Array::Array {
                 return Sidef::Types::Number::Number->new(-1);
             }
 
-            state $method = '=';
             foreach my $i (0 .. $#{$self}) {
-                $self->[$i]->$method($obj)
-                  && return Sidef::Types::Number::Number->new($i);
+                $self->[$i] eq $obj
+                  and return Sidef::Types::Number::Number->new($i);
             }
 
             return Sidef::Types::Number::Number->new(-1);
@@ -671,10 +747,9 @@ package Sidef::Types::Array::Array {
                 return Sidef::Types::Number::Number->new(-1);
             }
 
-            state $method = '=';
             for (my $i = $#{$self} ; $i >= 0 ; $i--) {
-                $self->[$i]->$method($obj)
-                  && return Sidef::Types::Number::Number->new($i);
+                $self->[$i] eq $obj
+                  and return Sidef::Types::Number::Number->new($i);
             }
 
             return Sidef::Types::Number::Number->new(-1);
@@ -741,8 +816,6 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Array::MultiArray->new($self, @args);
     }
 
-    *pairWith = \&pair_with;
-
     sub reduce {
         my ($self, $obj) = @_;
 
@@ -818,7 +891,6 @@ package Sidef::Types::Array::Array {
 
         my @sorted = do {
             my @arr;
-
             foreach my $i (0 .. $#{$self}) {
                 push @arr, [$i, $self->[$i]];
             }
@@ -911,7 +983,7 @@ package Sidef::Types::Array::Array {
         $self->new(grep { defined } @unique);
     }
 
-    *last_unique_by = \&uniq_by;
+    *last_unique_by = \&last_uniq_by;
 
     sub abbrev {
         my ($self, $code) = @_;
@@ -993,6 +1065,8 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Bool::Bool->false;
     }
 
+    *contain = \&contains;
+
     sub contains_type {
         my ($self, $obj) = @_;
 
@@ -1149,8 +1223,6 @@ package Sidef::Types::Array::Array {
         $self->new(@array);
     }
 
-    *joinInsert = \&join_insert;
-
     sub permute {
         my ($self, $code) = @_;
 
@@ -1203,7 +1275,7 @@ package Sidef::Types::Array::Array {
 
     sub unshift {
         my ($self, @args) = @_;
-        unshift @{$self}, @{$self->new(@args)};
+        CORE::unshift(@{$self}, @{$self->new(@args)});
         $self;
     }
 
@@ -1212,17 +1284,16 @@ package Sidef::Types::Array::Array {
     sub rotate {
         my ($self, $num) = @_;
 
-        $num = $num->get_value;
-        my $array = $self->new(@{$self});
+        $num = $num->get_value % ($#{$self} + 1);
+        return $self if $num == 0;
 
-        if ($num < 0) {
-            CORE::unshift(@{$array}, CORE::pop(@{$array})) for 1 .. abs($num);
-        }
-        else {
-            CORE::push(@{$array}, CORE::shift(@{$array})) for 1 .. $num;
-        }
+        # Surprisingly, this is slower:
+        # $self->new(@{$self}[$num .. $#{$self}], @{$self}[0 .. $num - 1]);
 
-        $array;
+        # Surprisingly, this is 73% faster:
+        my @array = @{$self};
+        CORE::unshift(@array, CORE::splice(@array, $num));
+        $self->new(@array);
     }
 
     # Join the array as string
@@ -1239,7 +1310,7 @@ package Sidef::Types::Array::Array {
 
     sub reverse {
         my ($self) = @_;
-        $self->new(reverse @{$self});
+        $self->new(CORE::reverse @{$self});
     }
 
     *reversed = \&reverse;    # alias
@@ -1288,7 +1359,7 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Bool::Bool->false;
     }
 
-    *remove_last = \&delete_first;
+    *remove_last = \&delete_last;
 
     sub delete {
         my ($self, $obj) = @_;
@@ -1334,6 +1405,22 @@ package Sidef::Types::Array::Array {
 
     *remove_first_if = \&delete_first_if;
 
+    sub delete_last_if {
+        my ($self, $code) = @_;
+
+        for (my $i = $#{$self} ; $i >= 0 ; --$i) {
+            my $item = $self->[$i];
+            $code->run($item) && do {
+                CORE::splice(@{$self}, $i, 1);
+                return Sidef::Types::Bool::Bool->true;
+              }
+        }
+
+        Sidef::Types::Bool::Bool->false;
+    }
+
+    *remove_last_if = \&delete_last_if;
+
     sub to_list {
         @{$_[0]};
     }
@@ -1380,13 +1467,13 @@ package Sidef::Types::Array::Array {
 
         *{__PACKAGE__ . '::' . '++'} = sub {
             my ($self, $obj) = @_;
-            $self->push($obj);
+            CORE::push(@{$self}, $obj);
             $self;
         };
 
         *{__PACKAGE__ . '::' . '--'} = sub {
             my ($self) = @_;
-            $self->pop;
+            CORE::pop(@{$self});
             $self;
         };
     }
