@@ -12,12 +12,12 @@ package Sidef::Types::Array::MultiArray {
 
     sub new {
         my (undef, @args) = @_;
-        my @array = map {
-            [map { $_->get_value } @{$_}]
-        } @args;
+        my @array = map { [@{$_}] } @args;
         bless \@array, __PACKAGE__;
     }
 
+    *call = \&new;
+
     sub get_value {
         my ($self) = @_;
         [
@@ -65,7 +65,7 @@ package Sidef::Types::Array::MultiArray {
 
     sub append {
         my ($self, $array) = @_;
-        push @{$self}, [map { $_->get_value } @{$array}];
+        push @{$self}, [@{$array}];
     }
 
     *push = \&append;
@@ -75,13 +75,12 @@ package Sidef::Types::Array::MultiArray {
         Sidef::Types::Array::Array->new(map { Sidef::Types::Array::Array->new(@{$_}) } @{$self});
     }
 
-    *to_a    = \&to_array;
-    *toArray = \&to_array;
+    *to_a = \&to_array;
 
     sub dump {
         my ($self) = @_;
         Sidef::Types::String::String->new(
-            'MultiArr.new(' . join(
+            'MultiArr(' . join(
                 ",\n\t     ",
                 map {
                     '[' . join(
