@@ -3,7 +3,10 @@ package Sidef::Types::Regex::Match {
     use 5.014;
     use overload
       q{bool} => \&to_bool,
-      q{""}   => \&to_s;
+      q{""}   => \&to_s,
+      q{@{}}  => sub {
+        $_[0]->{_cached_arr} //= [map { Sidef::Types::String::String->new($_) } @{$_[0]->{captures}}];
+      };
 
     use parent qw(
       Sidef::Object::Object
@@ -27,6 +30,7 @@ package Sidef::Types::Regex::Match {
                 $hash{matched} = 1;
             }
             else {
+                $hash{self}{pos} = 0;
                 $hash{matched} = 0;
             }
 
@@ -50,11 +54,11 @@ package Sidef::Types::Regex::Match {
 
         $hash{captures} = \@captures;
 
-        if (defined $hash{parser}) {
-            while (my ($key, $value) = each %{$hash{parser}{regexp_vars}}) {
-                $value->set_value(Sidef::Types::String::String->new($captures[$key - 1]));
-            }
-        }
+        #if (defined $hash{parser}) {
+        #    while (my ($key, $value) = each %{$hash{parser}{regexp_vars}}) {
+        #        $value->set_value(Sidef::Types::String::String->new($captures[$key - 1]));
+        #    }
+        #}
 
         bless \%hash, __PACKAGE__;
     }
@@ -85,12 +89,11 @@ package Sidef::Types::Regex::Match {
 
     sub named_captures {
         my ($self) = @_;
+
         my $hash = Sidef::Types::Hash::Hash->new();
-
-        foreach my $key (keys %{$self->{named_captures}}) {
-            $hash->{data}{$key} = Sidef::Types::String::String->new($self->{named_captures}{$key});
+        while (my ($key, $value) = each %{$self->{named_captures}}) {
+            $hash->{$key} = Sidef::Types::String::String->new($value);
         }
-
         $hash;
     }
 
