@@ -12,7 +12,7 @@ package Sidef::Types::Regex::Regex {
     use overload q{""} => \&get_value;
 
     sub new {
-        my (undef, $regex, $mode, $parser) = @_;
+        my (undef, $regex, $mode) = @_;
 
         if (ref($mode) eq 'Sidef::Types::String::String') {
             $mode = $mode->get_value;
@@ -30,7 +30,6 @@ package Sidef::Types::Regex::Regex {
                regex  => $compiled_re,
                global => $global_mode,
                pos    => 0,
-               parser => $parser,
               },
           __PACKAGE__;
     }
@@ -45,20 +44,21 @@ package Sidef::Types::Regex::Regex {
     sub match {
         my ($self, $object, $pos) = @_;
 
-        if (ref($object) eq 'Sidef::Types::Array::Array') {
+        $object //= Sidef::Types::String::String->new('');
+
+        if ($object->SUPER::isa('ARRAY')) {
             my $match;
             foreach my $item (@{$object}) {
-                $match = $self->match($item->get_value);
+                $match = $self->match($item);
                 $match->matched && return $match;
             }
             return $match // $self->match(Sidef::Types::String::String->new);
         }
 
         Sidef::Types::Regex::Match->new(
-                                        obj    => $object->get_value,
-                                        self   => $self,
-                                        parser => $self->{parser},
-                                        pos    => defined($pos) ? $pos->get_value : undef,
+                                        obj  => $object->get_value,
+                                        self => $self,
+                                        pos  => defined($pos) ? $pos->get_value : undef,
                                        );
     }
 
