@@ -37,26 +37,6 @@ package Sidef::Types::String::String {
 
     *to_str = \&to_s;
 
-    sub unroll_operator {
-        my ($self, $operator, $arg) = @_;
-        $self->to_chars->unroll_operator(
-
-            # The operator, followed by...
-            $operator,
-
-            # ...an optional argument
-            defined($arg)
-            ? $arg->to_chars
-            : ()
-
-        )->join;
-    }
-
-    sub reduce_operator {
-        my ($self, $operator) = @_;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } split(//, $$self))->reduce_operator($operator);
-    }
-
     sub inc {
         my ($self) = @_;
         my $copy = $$self;
@@ -244,11 +224,16 @@ package Sidef::Types::String::String {
 
     sub append {
         my ($self, $string) = @_;
-        __PACKAGE__->new($$self . $string->get_value);
+        $self->new($$self . $string->get_value);
     }
 
     *concat = \&append;
 
+    sub prepend {
+        my ($self, $string) = @_;
+        $self->new($string->get_value . $$self);
+    }
+
     sub ucfirst {
         my ($self) = @_;
         $self->new(CORE::ucfirst $$self);
@@ -898,18 +883,29 @@ package Sidef::Types::String::String {
     }
 
     sub contains {
-        my ($self, $string, $start_pos) = @_;
+        my ($self, $arg, $start_pos) = @_;
 
         $start_pos =
           defined($start_pos)
           ? $start_pos->get_value
           : 0;
 
+        if (ref($arg) eq 'Sidef::Types::Regex::Regex') {
+            my $regex = $arg->{regex};
+            my $s     = $$self;
+
+            if ($start_pos != 0) {
+                pos($s) = $start_pos;
+            }
+
+            return Sidef::Types::Bool::Bool->new(scalar $s =~ /$regex/g);
+        }
+
         if ($start_pos < 0) {
             $start_pos = CORE::length($$self) + $start_pos;
         }
 
-        Sidef::Types::Bool::Bool->new(CORE::index($$self, $string->get_value, $start_pos) != -1);
+        Sidef::Types::Bool::Bool->new(CORE::index($$self, $arg->get_value, $start_pos) != -1);
     }
 
     *include = \&contains;
@@ -1178,13 +1174,10 @@ package Sidef::Types::String::String {
                    and exists $chars[$i + 1]
                    and $chars[$i + 1] eq '{') {
                 if (ref $parser eq 'Sidef::Parser') {
-                    my $code = CORE::join('', @chars[$i + 1 .. $#chars]);
-                    my $block = $parser->parse_block(code => \$code);
-                    if (@{$block->{vars}} == 1) {
-                        $block = $block->{code};
-                    }
+                    my $code = 'do' . CORE::join('', @chars[$i + 1 .. $#chars]);
+                    my $block = $parser->parse_expr(code => \$code);
                     push @inline_expressions, [$i, $block];
-                    splice(@chars, $i--, 1 + pos($code));
+                    splice(@chars, $i--, 1 + pos($code) - 2);
                 }
                 else {
                     # Can't eval #{} at runtime!
@@ -1227,17 +1220,7 @@ package Sidef::Types::String::String {
             my $string = '';
             foreach my $char (@chars) {
                 if (ref($char)) {
-                    my $block =
-                      ref($char) eq 'HASH'
-                      ? $char
-                      : {
-                         $parser->{class} => [
-                                              {
-                                               self => $char,
-                                               call => [{method => 'run'}]
-                                              }
-                                             ]
-                        };
+                    my $block = $char;
 
                     if ($string ne '') {
                         $append_arg->(Encode::decode_utf8(Encode::encode_utf8($string)));
@@ -1306,7 +1289,6 @@ package Sidef::Types::String::String {
         *{__PACKAGE__ . '::' . '++'}  = \&inc;
         *{__PACKAGE__ . '::' . '-'}   = \&subtract;
         *{__PACKAGE__ . '::' . '=='}  = \&equals;
-        *{__PACKAGE__ . '::' . '='}   = \&equals;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '≠'} = \&ne;
         *{__PACKAGE__ . '::' . '>'}   = \&gt;
